package request

// ChartConfigReq
// @Description: 跨品种分析的图表配置
type ChartConfigReq struct {
	TagX           int               `description:"X轴的标签ID"`
	TagY           int               `description:"Y轴的标签ID"`
	CalculateValue int               `description:"计算窗口"`
	CalculateUnit  string            `description:"计算频度"`
	DateConfigList []ChartConfigDate `description:"日期配置列表"`
	VarietyList    []int             `description:"品种id列表"`
}

// ChartConfigDate
// @Description: 跨品种分析的日期配置
type ChartConfigDate struct {
	DateType int `description:"日期类型,，1：最新日期；2：N天前"`
	Num      int
}

// AddChartReq
// @Description: 添加图表的请求
type AddChartReq struct {
	ChartName      string            `description:"图表名称"`
	LeftMin        string            `description:"图表左侧最小值"`
	LeftMax        string            `description:"图表左侧最大值"`
	ChartImage     string            `description:"图表截图，复制的时候才用到" json:"-"`
	TagX           int               `description:"X轴的标签ID"`
	TagY           int               `description:"Y轴的标签ID"`
	CalculateValue int               `description:"计算窗口"`
	CalculateUnit  string            `description:"计算频度"`
	DateConfigList []ChartConfigDate `description:"日期配置列表"`
	VarietyList    []int

	// 主题相关
	ChartThemeId int    `description:"图表应用主题ID"`
	SourcesFrom  string `description:"图表来源"`
	Instructions string `description:"图表说明"`
	MarkersLines string `description:"标识线"`
	MarkersAreas string `description:"标识区"`
}

// EditChartReq
// @Description: 编辑图表的请求
type EditChartReq struct {
	ChartInfoId    int               `description:"图表id"`
	ChartName      string            `description:"图表名称"`
	LeftMin        string            `description:"图表左侧最小值"`
	LeftMax        string            `description:"图表左侧最大值"`
	ChartImage     string            `description:"图表截图，复制的时候才用到" json:"-"`
	TagX           int               `description:"X轴的标签ID"`
	TagY           int               `description:"Y轴的标签ID"`
	CalculateValue int               `description:"计算窗口"`
	CalculateUnit  string            `description:"计算频度"`
	DateConfigList []ChartConfigDate `description:"日期配置列表"`
	VarietyList    []int
}

// CopyAddChartInfoReq
// @Description: 复制并新增图表
type CopyAddChartInfoReq struct {
	ChartInfoId int    `description:"待复制的图表id"`
	ChartName   string `description:"图表名称"`
}

// EditChartEnInfoReq
// @Description: 编辑图表英文信息
type EditChartEnInfoReq struct {
	ChartInfoId int                `description:"图表ID"`
	ChartNameEn string             `description:"英文图表名称"`
	TagList     []TagNameEnReq     `description:"标签名称"`
	VarietyList []VarietyNameEnReq `description:"标签名称"`
}

// TagNameEnReq
// @Description: 标签英文名称修改
type TagNameEnReq struct {
	ChartTagId int    `json:"ChartTagId"`
	TagNameEn  string `json:"TagNameEn"`
}

// VarietyNameEnReq
// @Description: 品种英文名称修改
type VarietyNameEnReq struct {
	ChartVarietyId int    `json:"ChartVarietyId"`
	VarietyNameEn  string `json:"VarietyNameEn"`
}
